%% Mutual information and outage probability for Signal-Based Modulation
% calculates mutual information and probability of outage for MIMO-SBM
% constellation in Rayleigh fading channel + AWGN
clc
clear all

%%% Set input parameters
Nt = 1; % Number of transmit antennas
Nr = 2; % Number of receive antennas
R = 6; %  Outage probabilities are calculated at rates specified by array R
SNRdB = (25); % Signal to Noise Ratio(dB) at which the mutual information is calculated
SNR = 10 .^ (SNRdB/10); % Signal to Noise Ratio at which the mutual information is calculated
Es = Nt; % Transmit power equal to unity at each transmit antenna
N0 = Es ./ SNR; % White Gaussian noise spectral density
nSim = 1e4; % Number of realization of fading channel
%%
% Generate a realization of MIMO fading channel
generate_channel_matrix = @(Nr, Nt) sqrt(0.5) * (randn(Nr, Nt) + 1i* randn(Nr, Nt));
%%
% Calculate mutual information of the given channel realization
compute_mutual_info = @(Nr, Nt, H, N0) log2(det( eye(Nr) + (1/N0)*H*eye(Nt)*(H') ));
%%
%
% $$ I(H) = \log (\det(\mathbf{I}_{n_r} + \frac{1}{N_0} \mathbf{HKH^{\ast}}))$$ 
%
% $\mathbf{K}$ is the covriance matrix of transmit signal, $n_r$ number of transmit
% antennas
%

%% Generate different realization of fading channel and calculate MI for each realization
nSNR = numel(SNR);
I = zeros(nSNR, nSim);
for iSNR = 1 : nSNR
    parfor iSim = 1 : nSim
        if(~mod(iSim, 1e5))
            iSim
        end
        H = generate_channel_matrix(Nr, Nt);
        I(iSNR, iSim) = compute_mutual_info(Nr, Nt, H, N0);
    end
end

%% Calculate probability of outage
nR = numel(R);
pOutage = zeros(nSNR, nR);
for ir = 1 : nR
    % measure percentage of constellations falling below required rate
    pOutage(:, ir) = sum((I < R(ir)), 2);
end
pOutage = pOutage / nSim;

%% Save output avriables
%save('.\mat\sbm_MI_pOutage_mimo', 'I', 'SNRdB', 'pOutage')
